import os
import shutil
import tarfile
import zipfile
from os import makedirs
from os.path import join, isdir, exists, split

from coinlib.util import remove, get_package_cache_folder_path, download_package, copy_dir


def unpack_python_whl(whl_path, download_dir):

    unpack_root_dir = download_dir

    whl_dir, whl_filename = split(whl_path)
    whl_parts = whl_filename.split('-')
    unpack_dir_name = '%s-%s' % (whl_parts[0], whl_parts[1])
    unpack_dir = join(unpack_root_dir, unpack_dir_name)
    makedirs(unpack_dir)
    with zipfile.ZipFile(whl_path, "r") as z:
        z.extractall(unpack_dir)
    for item in os.listdir(unpack_dir):
        path = join(unpack_dir, item)
        if isdir(path) and item.endswith('.dist-info'):
            remove(path)
    return unpack_dir


def unpack_python_sources(archive_path, download_dir):
    unpack_dir = download_dir
    path, filename = split(archive_path)
    if filename.endswith('.tar.gz'):
        tarfile.open(archive_path).extractall(unpack_dir)
    else:
        with zipfile.ZipFile(archive_path, "r") as z:
            z.extractall(unpack_dir)
    sub_dirs = os.listdir(unpack_dir)
    if len(sub_dirs) != 1:
        raise Exception('Package %s is expected to contain one folder inside' % archive_path)
    subdir = sub_dirs[0]
    package_dir = join(unpack_dir, subdir)
    for item in os.listdir(package_dir):
        path = join(package_dir, item)
        if isdir(path):
            if item.endswith('.egg-info') or item in ['docs', 'tests', 'test']:
                remove(path)
        else:
            item_name, item_ext = os.path.splitext(item)
            useful_extensions = ['.py', '.pyc', 'pyo', '.pyd', '.dll', '.so']
            if item in ['setup.py', 'test.py', 'tests.py'] or item_ext not in useful_extensions:
                remove(path)
    return package_dir


def get_python_unpack_function(filename):
    unpack_function = None
    if filename.endswith('.whl'):
        unpack_function = unpack_python_whl
    if filename.endswith('.tar.gz') or filename.endswith('.zip'):
        unpack_function = unpack_python_sources
    if unpack_function is None:
        raise Exception('Unknown package type %s' % filename)
    return unpack_function


def install_python_package(cache_folder, url_or_path, ignore_cache, destination):
    package_path = url_or_path
    download_dir = get_package_cache_folder_path(cache_folder)
    if not exists(url_or_path):
        package_path = download_package(download_dir, url_or_path, ignore_cache)
    path, filename = split(package_path)
    unpack = get_python_unpack_function(filename)
    print("Package file: %s" % package_path)

    unpack_dir = unpack(package_path, download_dir)
    print("Unpacked to: %s" % unpack_dir)

    install_to = destination
    copy_dir(unpack_dir, install_to)
    print("Copied to: %s" % install_to)

    shutil.rmtree(unpack_dir, ignore_errors=True)
    print("Cleaned unpack dir: %s" % unpack_dir)



import os
import shutil
import tarfile
import zipfile
from os.path import join, exists, split

from coinlib.util import get_package_cache_folder_path, download_package, copy_dir


def just_unpack(archive_path, unpack_dir):
    path, filename = split(archive_path)
    if filename.endswith('.tar.gz') or filename.endswith('.tar.bz2'):
        tarfile.open(archive_path).extractall(unpack_dir)
    if filename.endswith('.zip'):
        with zipfile.ZipFile(archive_path, "r") as z:
            z.extractall(unpack_dir)


def unpack_raw(archive_path, download_dir, sub_folder):

    unpack_dir = download_dir
    if sub_folder is not None:
        unpack_dir = join(download_dir, sub_folder)
    just_unpack(archive_path, unpack_dir)

    sub_dirs = os.listdir(download_dir)
    if len(sub_dirs) != 1:
        raise Exception('Package {0} is expected to contain one folder inside {1}'.format(archive_path, download_dir))
    subdir = sub_dirs[0]

    package_dir = join(download_dir, subdir)
    return package_dir


def install_raw_package(cache_folder, url_or_path, ignore_cache, destination, sub_folder):
    package_path = url_or_path
    download_dir = get_package_cache_folder_path(cache_folder, sub_folder)
    if not exists(url_or_path):
        package_path = download_package(download_dir, url_or_path, ignore_cache)
    print("Package file: %s" % package_path)

    unpack_dir = unpack_raw(package_path, download_dir, sub_folder)
    print("Unpacked to: %s" % unpack_dir)

    copy_dir(unpack_dir, destination)
    print("Copied to: %s" % destination)



import os
import shutil
import subprocess
from os.path import join, exists

from coinlib.util import get_package_cache_folder_path, download_package, copy_dir


def dpkg_extract(archive_path, unpack_dir):
    if not exists(unpack_dir):
        os.mkdir(unpack_dir)
    subprocess.check_output(['dpkg', '-x', archive_path, unpack_dir])


def unpack_deb(archive_path, download_dir, sub_folder):
    unpack_dir = download_dir
    if sub_folder is not None:
        unpack_dir = join(download_dir, sub_folder)
    dpkg_extract(archive_path, unpack_dir)

    sub_dirs = os.listdir(download_dir)
    if len(sub_dirs) != 1:
        raise Exception('Package %s is expected to contain one folder inside' % archive_path)
    subdir = sub_dirs[0]

    package_dir = join(download_dir, subdir)
    return package_dir


def install_deb_package(cache_folder, url_or_path, ignore_cache, destination, sub_folder):
    package_path = url_or_path
    download_dir = get_package_cache_folder_path(cache_folder, sub_folder)
    if not exists(url_or_path):
        package_path = download_package(download_dir, url_or_path, ignore_cache)
    print("Package file: %s" % package_path)

    unpack_dir = unpack_deb(package_path, download_dir, sub_folder)
    print("Unpacked to: %s" % unpack_dir)

    install_to = destination
    copy_dir(unpack_dir, install_to)
    print("Copied to: %s" % install_to)

    shutil.rmtree(unpack_dir, ignore_errors=True)
    print("Cleaned unpack dir: %s" % unpack_dir)



from os.path import dirname, join

from coinlib.debian import install_deb_package
from coinlib.python import install_python_package
from coinlib.raw import install_raw_package
import pytest

DIR = dirname(__file__)
cache_dir = join(DIR, 'cache.dir')

@pytest.fixture(scope="function")
def fun_setup(request):
    if exists(cache_dir):
        shutil.rmtree(cache_dir)

def test_install_raw_package_explicit_cache():

    url_or_path = join(DIR, 'data', 'test.tar.gz')
    destination = join(DIR, 'install')

    install_raw_package(cache_dir, url_or_path, True, destination, 'test-raw')


def test_install_python_package_cache():

    url_or_path = join(DIR, 'data', 'test-1.whl')
    destination = join(DIR, 'install')

    install_python_package(cache_dir, url_or_path, True, destination)


def test_install_debian_package_cache():

    url_or_path = join(DIR, 'data', 'test.deb')
    destination = join(DIR, 'install')

    install_deb_package(cache_dir, url_or_path, True, destination, 'debtest')



import os
import shutil
import tempfile
import urllib
import urlparse
from os import makedirs
from os.path import basename, join, isdir, exists, split


def remove(path):
    if exists(path):
        if isdir(path):
            shutil.rmtree(path)
        else:
            os.remove(path)


def copy_dir(src, to):
    path, folder_name = split(src)
    target = join(to, folder_name)
    if exists(target):
        shutil.rmtree(target)
    shutil.copytree(src, target)


def download_package(download_dir, package_url, ignore_cache):
    filename = basename(urlparse.urlparse(package_url).path)
    download_path = join(download_dir, filename)

    if not exists(download_dir):
        makedirs(download_dir)

    if exists(download_path) and ignore_cache:
        remove(download_path)

    if not exists(download_path):
        print("Downloading: %s" % package_url)
        urllib.urlretrieve(package_url, filename=download_path)
    return download_path


def get_package_cache_folder_path(cache_dir, cache_folder=None):
    if not cache_dir:
        cache_dir = get_cache_dir()
    download_dir = cache_dir
    if cache_folder is not None:
        download_dir = join(download_dir, cache_folder)
    if not exists(download_dir):
        makedirs(download_dir)
    return download_dir


def get_cache_dir():
    temp_dir = tempfile.gettempdir()
    if temp_dir is None:
        temp_dir = os.getcwd()
    cache_dir = join(temp_dir, 'coin.cache')
    if not exists(cache_dir):
        makedirs(cache_dir)
    return cache_dir



from setuptools import setup
from os.path import join, dirname

requirements = [
    'urllib3==1.7.1'
]

version = open(join(dirname(__file__), 'version')).read().strip()
print(version)


setup(
    name='coin',
    version=version,
    py_modules=['coinlib/debian', 'coinlib/python', 'coinlib/raw', 'coinlib/util'],
    install_requires=requirements,
    scripts=['coin'],
    description='Copy Installer',
    license='GPLv3',
    author='Syncloud',
    author_email='support@syncloud.it',
    url='https://github.com/syncloud/coin'
)
